import java.lang.*;
import java.awt.*;
import java.applet.*;

/* There is exactly one abstract method here:
	
	void draw() 
	
  It draws the picture onto the offscreen image. */
  
abstract class PlotCanvas extends Canvas {
	PlotCTM ctm;
	Image img;
	Graphics gfx;
	int width = 0, height = 0;
	int borderwidth = 2;
	// different from background color
	Color canvascolour = Color.white;
	Applet parent;
	int px, py;

	public PlotCanvas(Applet app) {
		// Java 1.1: this(app, app.getBounds().width, app.getBounds().height);
		this(app, app.bounds().width, app.bounds().height);
	}

	public PlotCanvas(Applet app, int w, int h) {
		parent = app;
		setBackground(app.getBackground());
		width = w;
		height = h;
		ctm = new PlotCTM(width, height);
		img = parent.createImage(width, height);
		gfx = img.getGraphics();
		resize(width, height);
		clear();
	}
	
	// the default is to adjust for margin between parent and this
	public void adjustGfx() {
		int yOffset = bounds().y - parent.bounds().y;
		height -= yOffset;
		ctm = new PlotCTM(width, height);
		img = parent.createImage(width, height);
		gfx = img.getGraphics();
		resize(width, height);
		clear();
	}
	
	public void gsave() {
	
	}

	public void grestore() {
	
	}

	public void setBackground(Color c) {
		canvascolour = c;
	}

	public double pixelWidth() {
		return(1.0/ctm.sx);
	}
	
	public double pixelHeight() {
		return(1.0/ctm.sy);

	}

	void clear() {
		gfx.setColor(canvascolour);
		gfx.fillRect(0, 0, width, height);
		gfx.setColor(getForeground());
	}

	void clear(Graphics g) {
		g.setColor(canvascolour);
		g.fillRect(0, 0, width, height);
		g.setColor(getForeground());
	}
	
	// We follow Jim Morey in how paint and update are mangaged.
	
	public void paint(Graphics g) {
		update(g);
	}

	// If there are extras involved in drawing (say nodes)
	// this must be overridden

	public void update(Graphics g) {
		// clear the image
		clear();
		// draw the figure into the image
		draw();
		// map the image onto the screen:
		g.drawImage(img, 0, 0, null);
	}
	
	// this routine constructs the picture in the offscreen image
	abstract void draw();

	public boolean mouseDown(Event evt, int x, int y) {
		px = x; py = y;
		return false;
	}

	public boolean mouseUp(Event evt, int x, int y) {
		px = x; py = y;
		return false;
	} 

	public boolean mouseDrag(Event evt, int x, int y) {
		px = x; py = y;
		return false;
 	}
 	
	public static double interp(double t, double x, double y) {
		return((1.0-t)*x + t*y);
	}
	
	public void setCorners(double llx, double lly, double urx, double ury) {
		ctm.setCorners(llx, lly, urx, ury);
	}
	
	public void translate(double x, double y) {
		ctm.translate(x, y);
	}
	
	public void scale(double x, double y) {
		ctm.scale(x, y);
	}
	
	public Point toPoint(double x, double y) {
		return(ctm.toPoint(x, y));
	}

	public Point toPoint(RealPoint P) {
		return(ctm.toPoint(P));
	}

	public RealPoint toRealPoint(Point P) {
		return(ctm.toRealPoint(P));
	}

	public RealPoint toRealPoint(int x, int y) {
		return(ctm.toRealPoint(x, y));
	}
	
	public void drawString(String s, RealPoint Q, Color c) {
		Color tmp = gfx.getColor();
		gfx.setColor(c);
		Point P = toPoint(Q);
		gfx.drawString(s, P.x, P.y);
		gfx.setColor(tmp);
	}
	
	public void setColor(Color c) {
		gfx.setColor(c);
	}
	
	public void drawString(String s, double x, double y, Color c) {
		Color tmp = gfx.getColor();
		gfx.setColor(c);
		Point P = toPoint(x, y);
		gfx.drawString(s, P.x, P.y);
		gfx.setColor(tmp);
	}
	
	public void drawString(String s, RealPoint Q) {
		Point P = toPoint(Q);
		gfx.drawString(s, P.x, P.y);
	}
	
	public void drawString(String s, double x, double y) {
		Point P = toPoint(x, y);
		gfx.drawString(s, P.x, P.y);
	}
	
	double xmin() {
		double leftx = toRealPoint(0, 0).x;
		double rightx = toRealPoint(width, 0).x;
		if (leftx > rightx) return(rightx);
		return(leftx);
	}
	
	double xmax() {
		double leftx = toRealPoint(0, 0).x;
		double rightx = toRealPoint(width, 0).x;
		if (leftx > rightx) return(leftx);
		return(rightx);
	}
	
	double ymin() {
		double lefty = toRealPoint(0, 0).y;
		double righty = toRealPoint(width, 0).y;
		if (lefty > righty) return(righty);
		return(lefty);
	}
	double ymax() {
		double lefty = toRealPoint(0, 0).y;
		double righty = toRealPoint(width, 0).y;
		if (lefty > righty) return(righty);
		return(lefty);
	}
	
}
