import java.lang.*;
import java.awt.*;
import java.applet.*;

/* Each path is an array of segmentsa.
   Each segment is an array of coordinates.
   Stroking a path means drawing lines along the pieces.
   Filling it means turning each piece into a polygon. */
   
   // Design exceptions to recover from: no current point etc.
   
class PlotPath {
	final static int INC = 128;

	PlotCanvas canvas;
	Graphics gfx;
	Color colour = Color.black;

	PathSegment segment[];
	PathSegment currentSegment = null;
	int maxSegmentNo = INC;
	int segmentNo;
	
	// mark last move:
	int lastx, lasty;
	RealPoint CurrentPoint;
	int N = 0;

	public PlotPath(PlotCanvas can) {
		canvas = can;
		gfx = canvas.gfx;
		segment = new PathSegment[INC];
		segmentNo = 0;
	}

	public PlotPath(PlotCanvas can, Graphics g) {
		canvas = can;
		gfx = g;
		segment = new PathSegment[INC];
		segmentNo = 0;
	}

	public PlotPath(PlotCanvas can, Color col) {
		canvas = can;
		colour = col;
		segment = new PathSegment[INC];
		segmentNo = 0;
	}

	public void setColor(Color c) {
		colour = c;
	}

	RealPoint currentPoint() {
		return(CurrentPoint);
	}

	// - moves --------------------------------------------

	public void moveto(double x, double y) {
		if (segmentNo >= maxSegmentNo) {
			// System.out.println("overflow!");
			maxSegmentNo += INC;
			PathSegment newseg[] = new PathSegment[maxSegmentNo];
			for (int i=0;i<segmentNo;i++) {
				newseg[i] = segment[i];
			}
			newseg[segmentNo++] = new PathSegment();
			segment = newseg;
		}
		segment[segmentNo] = new PathSegment();
		currentSegment = segment[segmentNo++];
		Point q = canvas.ctm.toPoint(x, y);
		currentSegment.add(q);
		lastx = q.x; lasty = q.y;
		CurrentPoint = new RealPoint(x, y);
	}

	public void moveto(RealPoint p) {
		moveto(p.x, p.y);
	}
	
	public void rmoveto(RealPoint p) {
		double cx = CurrentPoint.x + p.x;
		double cy = CurrentPoint.y + p.y;
		moveto(cx, cy);
	}

	public void rmoveto(double x, double y) {
		double cx = CurrentPoint.x + x;
		double cy = CurrentPoint.y + y;
		moveto(cx, cy);
	}

	// - lines --------------------------------------------

	public void lineto(RealPoint p) {
		N++;
		Point q = canvas.ctm.toPoint(p);
		currentSegment.add(q);
		CurrentPoint.x = p.x;
		CurrentPoint.y = p.y;
	}
	
	public void lineto(double x, double y) {
		Point q = canvas.ctm.toPoint(x, y);
		N++;
		// System.out.println("lineto: q = " + q);
		currentSegment.add(q);
		CurrentPoint.x = x;
		CurrentPoint.y = y;
	}

	public void rlineto(RealPoint p) {
		CurrentPoint.x += p.x;
		CurrentPoint.y += p.y;
		N++;
		Point q = canvas.ctm.toPoint(CurrentPoint);
		currentSegment.add(q);
	}

	public void rlineto(double x, double y) {
		CurrentPoint.x += x;
		CurrentPoint.y += y;
		Point q = canvas.ctm.toPoint(CurrentPoint);
		N++;
		// System.out.println("lineto: q = " + q);
		currentSegment.add(q);
	}

	// - curves -------------------------------------------
	
	public void curveto(RealPoint P1, RealPoint P2, RealPoint P3) {
		P1 = canvas.ctm.toFinePoint(P1);
		P2 = canvas.ctm.toFinePoint(P2);
		P3 = canvas.ctm.toFinePoint(P3);
		curve.addCurve(currentSegment, P1, P2, P3);
	}

	// ----------------------------------------------------
	public void close() {
		currentSegment.add(lastx, lasty);
		// closed = true;
		moveto(lastx, lasty);
	}

	// all routines drawing a polygon return the polygon!
	// unlike PostScript - does not destroy the path
	// but we should close off graphics to it ...

	public void stroke() {
		stroke(colour);
	}

	public void stroke(Color c) {
		Color tmp = gfx.getColor();
		gfx.setColor(c);
		for (int i=0;i<segmentNo;i++) {
			PathSegment p = segment[i];
			for (int j=0;j<p.n-1;j++) {
				gfx.drawLine(p.x[j], p.y[j], p.x[j+1], p.y[j+1]);
			}
		}
		gfx.setColor(tmp);
	}

	public void fill() {
		// close path first?
		fill(colour);
	}

	public void fill(Color c) {
		// closepath first?
		Color tmp = gfx.getColor();
		gfx.setColor(c);
		for (int i=0;i<segmentNo;i++) {
			Polygon p = segment[i].toPolygon();
			gfx.fillPolygon(p);
		}
		gfx.setColor(tmp);
	}

	public void print() {
		for (int i=0;i<segmentNo;i++) {
			for (int j=0;j<segment[i].n;j++) {
				System.out.println(segment[i].x[j] 
					+ ", " 
					+ segment[i].y[j]);
			}
			System.out.println(":");
		}
	}
}

